class ParseError (Exception):pass

def __init__ (value,parseMemory):

    # Catch Error
    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    # Replace All The Symbols in The Var Name : @$^~:?
    value = value.replace('@','أ').replace('$','ب').replace('^','ت').replace('~','ث').replace('?','ج')
    
    # Variables that more than 1 Length :
    # my_name   or   my_age
    if (len(value)>2) or (len(value)==2 and not(value.startswith('#'))):
        # Replace From This         To This
        # my_var                    m_y___v_a_r
        value='_'.join(list(value))
    
    # 4h5  =  ID
    # #4  =  ID
    if value[0] in '1234567890' :
        value='_'+value
    
    # catch the words that used in class
    if (value == 'أ_c_o_n_s_t_r_u_c_t_o_r')    : value='__init__' 
    if (value == 'أ_s_t_r_i_n_g')    : value='__str__' 
    if (value == 'أ_r_e_p_r')    : value='__repr__' 
    if (value == 'أ_t_h_i_s')    : value='self' 

    # if function name not set yet
    if '|DATA_N|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA_N|',value)
    # if |DATA-P| not set yet
    elif '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|',value)
    # if |DATA| in line
    elif '|DATA|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA|',f'{value}|DATA|')
    # if var name not set yet
    elif '|DATA0|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA0|',value+'|DATA0|')
    # if this is var & is named
    elif '|DATA1|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA1|',f'{value}|DATA1|')
    elif parseMemory[5]=='':
        parseMemory[5]+=f'{value}|DATA|'
    else:
        raise ParseError(f'can\'t parsing (<file>, line {parseMemory[4]})')

    return parseMemory



def __init__ (value,parseMemory,addline=True):

    # catch some error
    # ( "|DATA|" , " |DATA|" ) Space before |..| means there is nothing added to data

    # while + for + if + elseif
    co_in_end = ('True ' in parseMemory[5]) or ('False ' in parseMemory[5]) or ('or ' in parseMemory[5]) or ('and ' in parseMemory[5]) or ('in ' in parseMemory[5])
    if (' |DATA|' in parseMemory[5]) and not(co_in_end) and (parseMemory[7] in ['while','for','if','elseif']) :
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\n{parseMemory[7]} sentence is not completed')
    
    # class + global + delvar
    if ('|DATA-P|' in parseMemory[5]) and (parseMemory[7] in ['class','global','delvar']):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\n{parseMemory[7]} sentence is not completed')
    
    # func name
    if (parseMemory[7] == 'func') and('|DATA_N|' in parseMemory[5]) :
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nfunction sentence is not completed')
    
    # else + try + catch
    if  (parseMemory[7] in ['else','try','catch']) and not(f'{parseMemory[7]} :'.replace('catch','except Exception as ERROR') == parseMemory[5]):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nunknown error in {parseMemory[7]} sentence')
    
    # break + continue
    if (parseMemory[5] in ['break','continue']) and not(parseMemory[7] == parseMemory[5]):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nunknown error in {parseMemory[7]}')
    
    # variable sentence
    var_has_not_items = ('= |DATA1|' in parseMemory[5]) or ('= str(|DATA1|' in parseMemory[5]) or ('= int(|DATA1|' in parseMemory[5]) or ('= float(|DATA1|' in parseMemory[5]) or ('= json.loads(|DATA1|' in parseMemory[5]) or ('= list(|DATA1|' in parseMemory[5])
       # variable didn't named yet                   # is var has an items     # var assign didn't writted
    if (parseMemory[5].startswith('|DATA0|')) or      var_has_not_items     or (not(parseMemory[6]) and ('|DATA1|' in parseMemory[5])):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nvariable sentence is not completed')
    

    if ('print(|DATA|,end=None)' in parseMemory[5]):
        parseMemory[5] = parseMemory[5].replace('|DATA|','\n')
    
    

    if parseMemory[1]:  # if There a Holder Add a new Block
        parseMemory[0]+=1
    parseMemory[1]=False
    parseMemory[6]=False
    # if There a Keyword Use a Code Block :
    # Add Line items to 'last_under_cmnd'
    if (parseMemory[7]==('while')) or parseMemory[7]==('for') or parseMemory[7]==('if') or parseMemory[7]==('elseif') or parseMemory[7]==('else') or parseMemory[7]==('def') or parseMemory[7]==('class') or parseMemory[7]==('try') or parseMemory[7]==('except') :
        parseMemory[2]=str(parseMemory[5])
    # Delete Somthings
    TheData= parseMemory[5].replace('|DATA|','').replace('|DATA1|','').replace('|DATA0|','').replace('|DATA-P|','').replace('|DATA_N|','')
    n=0
    for i in parseMemory[8]:
        TheData=TheData.replace(f'"[{n}]"',i)
        n+=1
    # Create Comment of The Kagsa Line in Python (this help to get the error from the compiler)
    if TheData.replace('\t','') !='':
        parseMemory[3].append('\t'*parseMemory[0] + TheData + f'    # line {parseMemory[4]}')
    parseMemory[5]='' # Clean Line items
    parseMemory[7]='' # Clean Command Type
    parseMemory[8]=[] # Clean Line Strings
    if addline:
        if value!=';':
            parseMemory[4]+=1 # Add New Line

    return parseMemory


from .methods import *
from .built_modules import *
from .errors import __init__ as errors
from .parse_id import __init__ as parse_id
from .paths import Paths as Paths
import sys,re,zipfile,traceback,os,requests,copy




def main (data,kagsa_file,lib=False,lib_name=None,memory=None):
    data = '\n'.join(data)
    # Replace ( Some Symbols Codes ) to ( Backslash + str sybmbols )
    data = data.replace('STR_SYM_TWO','\\"' ).replace('STR_SYM_ONE',"\\'" ).replace('STR_SYM_THREE',"``")
    #print(data)
    MEMORY = {
        'errors':errors,
        're':re,
        'os':os,
        'zipfile':zipfile,
        'sys':sys,
        'traceback':traceback,
        'parse_id':parse_id,
        'requests':requests,
        'Paths':Paths,
        'copy':copy
    }
    KAGSALINES=data
    kagsa_file = kagsa_file.replace('\\','\\\\')
    exec_data = Paths.getFile('exec_data.py','r').read().replace('[KAGSA-FILE]', kagsa_file)    
    data = exec_data + data
    if lib:
        nl='\n'
        kgl_text = Paths.getFile('libtemplate.py','r').read() + '\n' + f'KAGSA_FILE=Paths.__path__() + os.sep + "temp" + os.sep + "{kagsa_file}" \nKAGSA_FILE2 = "{kagsa_file}"\ntry: KAGSA_CODES=open(KAGSA_FILE).read() \nexcept: KAGSA_CODES=open(KAGSA_FILE2).read() \n' + data
        kgl = zipfile.ZipFile(lib_name,'w')
        kgl.writestr( kagsa_file , open(kagsa_file).read())
        kgl.writestr('main.py', kgl_text)
        kgl.close()
    # For Run a Code
    else:
        # 
        # Create a Memory for Run Python Codes
        #
        MEMORY = dict(globals() , **locals(), **MEMORY)
        if memory != None:
            for i,j in memory.items():
                MEMORY[i]=j
        MEMORY['KAGSA_CODES'] = KAGSALINES
        MEMORY['KAGSA_FILE']  = kagsa_file
        try:
            #print(KAGSALINES)
            exec(data,MEMORY)
            return MEMORY
        #
        # The Exception of Compiler Error
        #  How This Work ?
        #    1- Search For The Last Traceback
        #    2- Get The Line Thats Writed in The Translated Python Codes 
        #       ( The Line of Kagsa is Written in Pythhon Codes as a Comment )
        #    3- Read The File File and Get The Line
        #    4- Write Error
        #
        except Exception as ERR:
            #tb = ERR.__traceback__
            the_tb = traceback.extract_tb(sys.exc_info()[2])
            tb_filename, tb_lineno, tb_func, _ = the_tb[-1]
            tb_filetype = ''
            # if no lineno in the error frame
            #     go and get it from error text
            if tb_lineno=='':
                if ', line' in str(ERR):
                    tb_lineno = int(re.findall(r', line (\d+)',str(ERR))[0])
            
            # parse the error filename
            # error came from kagsa code
            if (tb_filename == '<string>') and (tb_func == '<module>'):
                tb_filename_print = kagsa_file
                tb_filetype = 'kg'
            # form a library
            elif 'kgtmp' in tb_filename:
                tb_filename_print = tb_filename.split(os.sep)[-1].replace('_main.py','')
                tb_filename_print = tb_filename_print.replace('__','ة').replace('_','').replace('ة','_')
                tb_filetype = 'py'
            # something else
            else:
                data_founded = False
                for ttb in the_tb:
                    tb_fn, tb_lno, tb_f, _ = ttb
                    if (tb_fn == '<string>') :
                        tb_filename, tb_lineno, tb_func, _ = ttb
                        data_founded = True
                        break
                if not(data_founded):
                    tb_lineno = int(re.findall(r', line (\d+)',str(ERR))[0])
                tb_filename_print = kagsa_file
                tb_filetype = 'kg'
            


            
            try:                
                if tb_filetype == 'kg':
                    tb_lineno = data.split('\n')[tb_lineno-1]
                    tb_lineno = int(re.findall(r'    # line (\d+)',tb_lineno)[-1])
                    file_lines = open(kagsa_file,'r').read().split('\n')
                    file_lines_no = len( file_lines )
                else:
                    file_lines = open(tb_filename, 'r').read().split('\n')
                    file_lines_no = len( file_lines )
                
                ########

                if (tb_lineno == file_lines_no) or (tb_lineno < file_lines_no):
                    line_text = file_lines[tb_lineno-1].strip()
                    #tb_lineno   = tb_lineno
                else:
                    line_text = '?'
                    tb_lineno   ='?'
            except:
                line_text = '?'
                tb_lineno   ='?'
            
            tb_type, tb_text, _ = errors(ERR, lineno=str(tb_lineno),get_value_back=1)

            # tb_filetype
            # line_text
            # tb_lineno
            # tb_filename_print
            # tb_type
            # tb_text

            # Colors :
            def Red (t) : return f'\x1b[1;31m{t}\x1b[0m'
            def Cyan (t) : return f'\x1b[0;36m{t}\x1b[0m'
            def Yellow (t) : return f'\x1b[0;33m{t}\x1b[0m'

            # Start Print The Error
            # print first line : error catched [ .. ]
            if tb_filetype == 'py':
                print(f'{Red("error catched [")} {tb_filename_print}.kgl/main.py/{tb_type}{Red(" ]")}')
            else:
                print(f'{Red("error catched [")} {tb_filename_print}/{tb_type}{Red(" ]")}')
            # print the line text/no of error
            print(' ' + (' '* len(str(tb_lineno))) + ' |')
            print(f' {Cyan(tb_lineno)} | {Cyan(line_text)}')
            print(' ' + (' '* len(str(tb_lineno))) + ' |')
            # print err string with the details
            if '\n' in tb_text:
                print(f'{Red("error")}:', tb_text.split('\n')[0]) # error
                print(f'{Yellow("details")}:', '\n'.join(tb_text.split('\n')[1:])) # details
            else:
                print(f'{Red("error")}:', tb_text) # error

        except KeyboardInterrupt:
            print('\n\nCTRL + C')


from .lexer import main as Lexer
from .paths import Paths as Paths
import re


def __init__ (value):
    parse_string = Paths.getFile('parse_string.py','r').read().replace('from .parse_sformat import __init__ as parse_sformat','')
    space = dict(
        parse_sformat=globals()['__init__'],
        **globals()
    )
    exec(parse_string,space)
    parse_string = space['__init__']
    space = dict(
        parse_sformat=globals()['__init__'],
        parse_string=parse_string,
        **globals()
    )
    parser = Paths.getFile('parser.py','r').read().replace('from .parse_string import __init__ as parse_string','')
    exec(parser,space)
    lex   = Lexer(value)
    parse = space['main'](lex)
    parse = '\n'.join(parse)
    parse = parse.replace('STR_SYM_TWO','\\"' ).replace('STR_SYM_ONE',"\\'" ).replace('STR_SYM_THREE',"``")
    parse = parse.replace(re.findall(r'    # line \d+',parse)[0],'')
    return parse.replace('\n','')


class ParseError (Exception):pass


def __init__ (value,parseMemory):

    # catch errors
    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    if value.replace('0','') != '':
        while True:
            if value.startswith('0'):
                value = value[1:]
            else:
                break
        if value=='':
            raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nNumber Can\'t be zero')

    # if |DATA-P| not set yet    
    if '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|',value)
    # if |DATA| in line
    elif '|DATA|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA|',f'{value}|DATA|')
    # var item bar found
    elif '|DATA0|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA0|',f'{value}|DATA0|')
    elif '|DATA1|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA1|',f'{value}|DATA1|')
    elif parseMemory[5]=='':
        parseMemory[5]+=f'{value}|DATA|'
    else:
        raise ParseError(f'can\'t parsing (<file>, line {parseMemory[4]})')
    return parseMemory


from .parse_endl import __init__ as parse_endl
from .parse_cmnt import __init__ as parse_cmnt
from .parse_keyword import __init__ as parse_keyword
from .parse_id import __init__ as parse_id
from .parse_dot_coma import __init__ as parse_dot_coma
from .parse_number import __init__ as parse_number
from .parse_assign import __init__ as parse_assign
from .parse_string import __init__ as parse_string
from .parse_math import __init__ as parse_math
from .parse_paren import __init__ as parse_paren
from .parse_cparen import __init__ as parse_cparen
from .parse_co import __init__ as parse_co

class ParseError (Exception) :pass

def main (data):
    parseMemory = [
        0,       # under_codes        : [0] : Current Codes Blocks (Tab)
        False,   # under_codes_holder : [1] : Holder to get a New Block ('if' ,'else' ,'while')
        '',      # last_under_cmnd    : [2] : The Holder Value
        [],      # parsed_input       : [3] : All The Translated Python Codes (list)
        1,       # line               : [4] : The Current Line (used to print in errors)
        '',      # line_items         : [5] : The Line Items (its will be edit and add it to 'parsed_input')
        False,   # variable close     : [6] : Is The Variable Assign Writted ?
        '',      # command type       : [7] : The Type of the Command in The Line (write,var,int,try)
        []       # strings            : [8] : All The Strings That Used in The Line
    ]
    for key,value in data:          # Read All The Lexed Data ( [['key','value']..] )
        if    key == 'ENDLINE':
            parseMemory = parse_endl(value,parseMemory)
        elif  key == 'COMMENT':
            parseMemory = parse_cmnt(value,parseMemory)
        elif  key == 'KEYWORD':
            parseMemory = parse_keyword(value,parseMemory)
        elif  key == 'ID':
            parseMemory = parse_id(value,parseMemory)
        elif (key == 'COMA') or (key == 'DOT'):
            parseMemory = parse_dot_coma(value,parseMemory)
        elif (key == 'INT') or (key == 'FLOAT'):
            parseMemory = parse_number(value,parseMemory)
        elif (key == 'ASSIGN'):
            parseMemory = parse_assign(value,parseMemory)
        elif  key == 'STRING':
            parseMemory = parse_string(value,parseMemory)
        elif  key=='MATH':
            parseMemory = parse_math(value,parseMemory)
        elif (key=='RPAREN') or(key=='LPAREN'):
            parseMemory = parse_paren(value,parseMemory)
        elif (key=='RCPAREN') or(key=='LCPAREN'):
            parseMemory = parse_cparen(value,parseMemory)
        elif  key=='CO':
            parseMemory = parse_co(value,parseMemory)
    #print(parseMemory[3])
    return parseMemory[3]



def __init__ (value, parseMemory):

    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    if '|DATA_N|' in parseMemory[5]:
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nComplete function sentence')
    if '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|','')
    if '|DATA|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA|',f'{value}|DATA|')
    elif '|DATA0|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA0|',f'{value}|DATA0|')
    elif '|DATA1|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA1|',f'{value}|DATA1|')
    else:
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nUnknown place of paren')
    
    return parseMemory


class ParseError(Exception):pass

def __init__ (value,parseMemory):

    # errors
    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    value=value.replace(' ','')
    newValues={
        '->':' in ',
        '<=':'<=',
        '>=':'>=',
        '==':'==',
        '<':'<',
        '>':'>',
        '!=':'!=',
        '||':' or ',
        '&&':' and ',
        'true':' True ',
        'false':' False '
    }
    value = newValues[value]

    # if |DATA-P| not set yet
    if '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|',value)
    # if |DATA| in line
    elif '|DATA|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA|',f'{value}|DATA|')
    # if var name not set yet
    elif '|DATA0|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA0|',f'{value}|DATA0|')
    # if this is var & is named
    elif '|DATA1|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA1|',f'{value}|DATA1|')
    elif parseMemory[5]=='':
        parseMemory[5]+=f'{value}|DATA|'
    else:
        raise ParseError(f'can\'t parsing (<file>, line {parseMemory[4]})')
    
    return parseMemory


class ParseError (Exception):pass

def __init__ (value,parseMemory):

    # catch error
    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    # if |DATA-P| is empty : close it
    if '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|','')
    # if |DATA| found
    if '|DATA|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA|',f'{value}|DATA|')
    #
    elif ('|DATA1|' in parseMemory[5]) and (parseMemory[6] == False):
        parseMemory[5]=parseMemory[5].replace('|DATA0|',f'{value}|DATA0|')
    # var items bar is opened
    elif ('|DATA1|' in parseMemory[5]):
        parseMemory[5]=parseMemory[5].replace('|DATA1|',f'{value}|DATA1|')
    else:
        raise ParseError(f'can\'t parsing (<file>, line {parseMemory[4]})')
    
    return parseMemory


import re
def main (data):
    opened_paren=0
    KEYS = ""
    for d in data:
        current_line = KEYS.count("\n")
        # if The TYPE is End Line -> Close The Current Line & Open New One
        # And Check if The Opened Paren == 0
        if d[0] == 'ENDLINE':
            if d[1]!=';' :
                KEYS+='.ENDLINE\n'
            else:
                KEYS+='.SEMICOLON'
            if opened_paren != 0:
                raise SyntaxError(f'invalid syntax (<file>, line {current_line})\nLine ended without close the paren')
            continue
        
        if d[0] == 'COMMENT':
            lines=d[1].count('\n')
            data = '.ENDLINE\n' * lines
            KEYS+=data
        # if There a Open Paren Add 1 to ( opened_paren )
        if d[0]=='LPAREN': # (
            opened_paren+=1
            KEYS+='.'+d[0]
            continue
        
        # if There a Open Paren Minus 1 From ( opened_paren )
        if d[0]=='RPAREN': # )
            if opened_paren == 0:
                raise SyntaxError(f'invalid syntax (<file>, line {current_line})\nLine ended without close the paren')
            opened_paren-=1
            KEYS+='.'+d[0]
            continue
        
        # Add TYPE to a String With (.) Between Them
        if (d[0]=='PLUS') or (d[0]=='MINUS') or (d[0]=='TIMES') or (d[0]=='DIVIDE') :
            KEYS+='.MATH'
        if (d[0]=='KEYWORD' and d[1]=='input'):
            KEYS+='.INPUT'
        if (d[0]=='STRING' and d[1].startswith('``')):
            KEYS+='.'+d[0]
            #current_line+=(d[1].count('\n'))
            #for i in range(1,d[1].count('\n')+1):
            #    lines[str(1)]=''
            lines=d[1].count('\n')
            data = '.ENDLINE\n' * lines
            KEYS+='.'+data
        else:
            KEYS+='.'+d[0]
    
    WrongsSyntax=[
        r'KEYWORD.KEYWORD',
        r'MATH.KEYWORD',
        r'ASSING.KEYWORD',
        r'INT.KEYWORD',
        r'STRING.KEYWORD',
        r'FLOAT.KEYWORD',
        r'CO.KEYWORD',
        r'COMA.KEYWORD',
        r'KEYWORD.ASSING',
        r'KEYWORD.MATH',
        r'STRING.MATH.INT',
        r'INT.MATH.STRING',
        r'STRING.MATH.FLOAT',
        r'FLOAT.MATH.STRING',
        r'STRING.INT',
        r'STRING.FLOAT',
        r'INT.STRING',
        r'FLOAT.STRING',
        r'INT.FLOAT',
        r'FLOAT.INT',
        r'INT.INT',
        r'STRING.STRING',
        r'FLOAT.FLOAT',
        r'DOT.DOT',
        r'MATH.MATH',
        r'COMA.COMA',
        r'STRING.ID',
        r'ID.ID',
        r'INT.ID',
        r'FLOAT.ID',
        r'ASSIGN.COMA',
        r'MATH.COMA',
        r'COMA.ASSIGN',
        r'COMMA.MATH',
        r'LCPAREN.[\n-\.ENDLINE-ENDLINE]*.RCPAREN',
        r'LCPAREN.ENDLINE.RCPAREN'
    ]
    # Checking The Wrong Syntax
    #print(KEYS)
    for w in WrongsSyntax:
        searchForError=re.findall(w,KEYS)
        if len(searchForError) > 0:
            #print(w)
            line = KEYS[  0 :  KEYS.find(searchForError[0])   ].count('\n') + 1
            raise SyntaxError(f'invalid syntax (<file>, line {line})\nError code : {w.lower()}')

    return 1


import re


def __init__ (theErr,get_value_back=False, lineno=None):
    ErrStr=str(theErr).replace('<string>','<file>')
    ErrStr=ErrStr.replace('expected an indented block after function definition' , 'empty code block')
    ErrStr=ErrStr.replace('print()' , 'write')
    ErrStr=ErrStr.replace('str()' , 'string variable')
    ErrStr=ErrStr.replace('float()' , 'float variable')
    ErrStr=ErrStr.replace('int()' , 'int variable')
    ErrStr=ErrStr.replace('JSONDecoder.__init__()' , 'json variable')
    ErrStr=ErrStr.replace('list()' , 'list variable')
    ErrStr=ErrStr.replace('input()' , 'input')
    ErrStr=ErrStr.replace('INCLUDE()' , 'include')
    ErrStr=ErrStr.replace('JUMP()' , 'jump')
    ErrStr=ErrStr.replace('STR_SYM_TWO','\\"').replace("STR_SYM_ONE","\\'").replace("STR_SYM_THREE","\\``")
    ErrType=theErr.__class__.__name__.replace('error','ERR').replace('Error','ERR').replace('compiler.classes.__init__.','')
    if lineno!=None:
        lll = re.findall(r'line \d+',ErrStr)
        for i in lll: ErrStr=ErrStr.replace(i , 'line '+lineno)
    #############################################
    Symbols = {
        'أ' : '@',
        'ب' : '$',
        'ت' : '^',
        'ث' : '~',
        'ج' : '?'
    }
    for i in Symbols.items():
        ErrStr = ErrStr.replace(i[0], i[1])
        ErrType = ErrType.replace(i[0], i[1])
    #############################################
    ErrStr_tmp = ErrStr.replace('(','').replace(')','').replace('\'','').replace('"','').replace('.',' . ')
    ErrType_tmp = ErrType.replace('.',' . ')
    try:
        for word in [word for word in ErrStr_tmp.split(' ') if not(word[0].isdigit() or word[0] == '_' and not word[1].isdigit() or any([len(char) > 1 for char in word.split('_')]))]:
            new_word = ''.join(word.replace('__','ظ').split('_')).replace('ظ','_')
            ErrStr = ErrStr.replace(word,new_word)
        
        for word in [word for word in ErrType_tmp.split(' ') if not(word[0].isdigit() or word[0] == '_' and not word[1].isdigit() or any([len(char) > 1 for char in word.split('_')]))]:
            new_word = ''.join(word.replace('__','ظ').split('_')).replace('ظ','_')
            ErrStr = ErrStr.replace(word,new_word)
    except:None
    #############################################
    if get_value_back:
        return [ErrType, ErrStr, f'{ErrType} : {ErrStr}']
    else:
        print(f'{ErrType} : {ErrStr}',end='')


from sly import Lexer
import re

class Lexer(Lexer):
    # Tokens Names
    tokens = { COMMENT, KEYWORD, CO, ID, DOT, FLOAT, INT, STRING, MATH, ASSIGN, 
        LPAREN, RPAREN, LCPAREN, RCPAREN, ENDLINE, COMA}

    # Ignored Characters Between Tokens
    ignore = ' \t'

    # Regex Rules
    COMMENT    = r'//..*|/\*(.*?|\n)*\*/|//|/\**\*'
    #KEYWORD    = r'input|write|var|if|elseif|else|while|func|break|return|global|for|try|catch|delvar|continue|class'
    # Search for Keyword will be dwon in run func (for loop)
    # this will fix this problem
    # var vari = ''
    CO         = r'true|false|->|==|>|<|<=|>=|!=|\|\||&&|= *=|>|<|< *=|> *=|! *=|\| *\||& *&|- *>'
    ID         = r'[0-9a-zA-Z_@$^~?][0-9a-zA-Z_@$^~?]*|[0-9a-zA-Z_@$^~?]'
    DOT        = r'\.'
    FLOAT      = r'\d+\.\d+'
    INT        = r'\d+|\-\d+'
    #            | Math on Var            | Math on Var (with space)            | Math Symbols
    MATH       = r'\*\*=|\+=|\-=|/=|\*=|%=|\* *\* *=|\+ *=|\- *=|/ *=|\* *=|% *=|\*\*|\* *\*|\+|\-|%|/|\*'
    ASSIGN     = r'='
    LPAREN     = r'\('
    RPAREN     = r'\)'
    LCPAREN    = r'\{'
    RCPAREN    = r'\}'
    STRING     = r'\'(.*?)\'|"(.*?)"|``(.*?|\n)*``'
    ENDLINE    = r'\n|;'
    COMA       = r','


def main (context) :
    context=context.replace('\\"','STR_SYM_TWO')
    context=context.replace("\\'",'STR_SYM_ONE')
    context=context.replace("\\``",'STR_SYM_THREE')
    lexer = Lexer()
    try:
        end=[]
        # Start Lexeing Loop and Get Outout
        for tok in lexer.tokenize(context):
            if tok.type == 'ID' and tok.value in ['jump','include','input','write','var','if','elseif','else','while','func','break','return','global','for','try','catch','delvar','continue','class']:
                tok.type = 'KEYWORD'
            if (tok.type == 'ID') and (tok.value in ['string','int','float','list','dict']):
                if len(end) > 1 :
                    if (end[-1][0] == 'ENDLINE'):
                        tok.type = 'KEYWORD'
                else:
                    tok.type = 'KEYWORD'
            # myfile = File.write('f.txt');
            # Write Func     ^
            # Will Be Lexed as ID
            if (tok.type=='KEYWORD') and (len(end) > 2):
                if (end[-1][0] == 'DOT') or (end[-1][0] == 'LPAREN') or (end[-1][0] == 'COMA'):
                    if tok.value != 'input':
                        tok.type = 'ID'
            # if true {   write 'ok\n';   } else{   write 'err\n';   }
            if tok.type=='RCPAREN':
                end.append(['ENDLINE',';'])
                end.append(['RCPAREN','}'])
                end.append(['ENDLINE',';'])
            if tok.type=='LCPAREN':
                end.append(['ENDLINE',';'])
                end.append(['LCPAREN','{'])
                end.append(['ENDLINE',';'])
            #   #0  = ID
            #   454 = INT
            #   5h4 = ID
            if tok.type == 'ID':
                if re.sub(r'[0-9]*','',tok.value) == '':
                    end.append(['INT',tok.value])
                else:
                    end.append([tok.type,tok.value])
            else:
                end.append([tok.type,tok.value])
        # End Loop
        end.append(['ENDLINE','\n'])
        #print(end)
        return end
    except Exception as err:
        # Return a Illegal Character Error
        if 'Illegal character' in str(err):
            ERR=str(err)
            # Get the illegal char
            try:
                char=re.findall(r"'(.*?)'",ERR)[0]
            except:
                char=re.findall(r'"(.*?)"',ERR)[0]
            # Get the index
            index=int(ERR[ERR.find('index '):].replace('index ',''))
            # Get the line
            line=context[0:index].count('\n')+1
            raise SyntaxError(f'illegal char "{char}" in index {index}, line {line}')
        # Return a Unknown Error
        else:
            raise SyntaxError(str(err))


from .parse_id import __init__ as parse_id
from .parse_sformat import __init__ as parse_sformat
import re
class ParseError (Exception) :pass

def __init__ (value,parseMemory):

    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    # parse the string that have more than 1 line
    if value.startswith('``'):
        value='"""'+value[2:-2]+'"""'
        parseMemory[4]+=value.count('\n')
    
    # Check Formats
    FORMAT=re.findall(r'(%{(.*?)})',value)
    if len(FORMAT) > 0:
        value='f'+value
        for d in FORMAT:
            # d[0] : %{..}
            # d[1] :   ..
            format_id = parse_sformat(d[1])
            NEW='{'+format_id+'}'
            value=value.replace(d[0],NEW)
    
    value=value.replace('STR_SYM_ONE',"\\'").replace('STR_SYM_TWO','\\"')
    string_idx = len(parseMemory[8])
    if '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|',f'"[{string_idx}]"')
    elif '|DATA|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA|',f'"[{string_idx}]"|DATA|')
    elif ('|DATA0|' in parseMemory[5]):
        parseMemory[5]=parseMemory[5].replace('|DATA0|',f'"[{string_idx}]"|DATA0|')
    elif ('|DATA1|' in parseMemory[5]):
        parseMemory[5]=parseMemory[5].replace('|DATA1|',f'"[{string_idx}]"|DATA1|')
    elif parseMemory[5]=='':
        parseMemory[5]+=f'"[{string_idx}]"|DATA|'
    else:
        raise ParseError(f'can\'t parsing (<file>, line {parseMemory[4]})')
    parseMemory[8].append(value)
    #print(parseMemory[5])
    return parseMemory


import sys,os

# from .paths import Paths as Paths
# Paths.init()
# Paths.getFile(  )

class Paths:
    def __path__ ():
        if getattr(sys, 'frozen', False):
            return os.path.dirname(sys.executable)
        # or a script file (e.g. `.py` / `.pyw`)
        elif __file__:
            return os.path.dirname(__file__)

    def init () :
        sys.path.insert(1, os.getcwd() )
        sys.path.insert(1, Paths.__path__() )

    def getFile (filen,mode):
        '''if Paths.AsPackage:
            pkg_path = os.path.dirname( pkg_resources.resource_stream(__name__,'errors.py').name ) + os.sep
            return open(pkg_path + filen,mode,encoding='utf-8')
        else:
            return open( Paths.__path__() + os.sep + filen, mode , encoding='utf-8')'''
        return open( Paths.__path__() + os.sep + filen, mode , encoding='utf-8')


#pkg_resources.resource_stream(__name__,'built_modules.py').read().decode('utf-8')


#!/usr/bin/env python3
import sys,os
from .paths import Paths as Paths

Paths.init()
if not('temp' in os.listdir(Paths.__path__())):
    os.mkdir(Paths.__path__()+os.sep+'temp')
if not('libs' in os.listdir(Paths.__path__())):
    os.mkdir(Paths.__path__()+os.sep+'libs')
sys.path.insert(0,Paths.__path__() + os.sep + 'libs')
sys.path.insert(0,Paths.__path__() + os.sep + 'temp')

from .lexer import main as Lexer
from .syntax_checker import main as Syntax_checker
from .parser import main as Parser
from .compiler import main as Compiler
from .errors import __init__ as errors
import platform,requests,re,logging,coloredlogs,shutil


def main ():
    pass

if platform.system() == 'Windows' :
    logger = logging.getLogger(f"Logger")
    coloredlogs.install(logger=logger)

version = '1.2.0'

# Colors :
def Red (t) : return f'\x1b[1;31m{t}\x1b[0m'
def Cyan (t) : return f'\x1b[0;36m{t}\x1b[0m'
def Yellow (t) : return f'\x1b[0;33m{t}\x1b[0m'

def ArgsParser ():
    argv = ' '.join(sys.argv[1:])
    data = {'do':''}
    if re.search(r"(-us *\S+)|(--unsetup *\S+)",argv):
        if not(argv in re.findall(r"(-us *\S+)|(--unsetup *\S+)",argv)[0]):
            return {'do':''}
        data['do'] = 'unsetup_kgl'
        data['state'] = 'ok'
        data['message'] = ''
        if not(sys.argv[2].endswith('.kgl')):
            data['state'] = 'err'
            data['message'] = 'error : unsetup_kgl : input file must be .kgl'
            return data
        data['input'] = [sys.argv[2]]
        return data
    elif   re.search(r'-v|--version',argv):
        if re.findall(r'-v|--version',argv)[0] != argv:
            return {'do':''}
        data['do'] = 'check_version'
    elif re.search(r'-u|--updates',argv):
        if re.findall(r'-u|--updates',argv)[0] != argv:
            return {'do':''}
        data['do'] = 'check_updates'
    elif re.search(r'-h|--help',argv):
        if re.findall(r'-h|--help',argv)[0] != argv:
            return {'do':''}
        data['do'] = 'help'
    elif argv=='':
        data['do'] = 'shell'
    elif re.search(r"(-l *\S+ *-o *\S+)|(--lib *\S+ *--out *\S+)",argv):
        if not(argv in re.findall(r"(-l *\S+ *-o *\S+)|(--lib *\S+ *--out *\S+)",argv)[0]):
            return {'do':''}
        data['do'] = 'compile_lib'
        data['state'] = 'ok'
        data['message'] = ''
        if not(sys.argv[2].endswith('.kg')) or not(sys.argv[4].endswith('.kgl')):
            data['state'] = 'err'
            data['message'] = 'error : compile_lib : input file must be .kg, output file must be .kgl'
            return data
        data['input'] = [sys.argv[2], sys.argv[4]]
        return data
    elif re.search(r"(-s *\S+)|(--setup *\S+)",argv):
        if not(argv in re.findall(r"(-s *\S+)|(--setup *\S+)",argv)[0]):
            return {'do':''}
        data['do'] = 'setup_kgl'
        data['state'] = 'ok'
        data['message'] = ''
        if not(sys.argv[2].endswith('.kgl')):
            data['state'] = 'err'
            data['message'] = 'error : setup_kgl : input file must be .kgl'
            return data
        data['input'] = [sys.argv[2]]
        return data
    elif re.search(r'-ct|--cleartmp',argv):
        if re.findall(r'-ct|--cleartmp',argv)[0] != argv:
            return {'do':''}
        data['do'] = 'cleartmp'
    else:
        if argv.endswith('.kg'):
            data['do'] = 'run_file'
            data['state'] = 'ok'
            data['message'] = ''
            if len(sys.argv[1:]) != 1:
                data['state'] = 'err'
                data['message'] = 'error : run_file : run file command must be like "kagsa file.kg"'
                return data
            data['input'] = [argv]
            return data
    return data


args = ArgsParser()
# Run a Code
# kagsa File.kg
if (args['do'] == 'run_file'):
    if args['state'] != 'ok':
        sys.exit(args['message'])
    
    kg_file = args['input'][0]
    ## Check main file
    try:
        open(kg_file,'r')
    except:
        sys.exit(f'error : run_file : "{kg_file}" is not defined')

    error_in = None # used to catch the error
    try:
        error_in='Lexer'
        LEXER = Lexer(  open(kg_file,'r').read()  )
        error_in='SyntaxChecker'
        SYNTAX = Syntax_checker(LEXER)
        error_in='Parser'
        PARSER = Parser(LEXER)
        error_in=None
    except Exception as ee:
        if ', line' in str(ee):
            line_no=int(re.findall(r', line (\d+)',str(ee))[0])
            line_text=open(kg_file,'r').read().split('\n')[line_no-1].strip()
        else:
            line_no = '?'
            line_text = '?'
        tb_type, tb_text, _ = errors(ee, lineno=str(line_no),get_value_back=1)

        print(f'{Red("error catched [")} {error_in}/{kg_file}/{tb_type} {Red("]")}')
        print(' ' + (' '* len(str(line_no))) + ' |')
        print(f' {Cyan(line_no)} | {Cyan(line_text)}')
        print(' ' + (' '* len(str(line_no))) + ' |')
        if '\n' in tb_text:
            print(f'{Red("error")}:', tb_text.split('\n')[0]) # error
            print(f'{Yellow("details")}:', '\n'.join(tb_text.split('\n')[1:])) # details
        else:
            print(f'{Red("error")}:', tb_text) # error


    if error_in==None:
        Compiler(PARSER,kg_file)
# Run Kagsa without any Command
# Start Kagsa Console
elif (args['do'] == 'shell'):
    memory= {}
    first_time_run_command = True
    print(f'KAGSA PROGRAMMING LANGUAGE {version}\nMIT License: Copyright (c) 2022 Kagsa',end='')
    while True:
        try:
            command = input('\n>>>')
            error_in = None # used to catch the error
            try:
                error_in='Lexer'
                LEXER = Lexer(  command  )
                error_in='SyntaxChecker'
                SYNTAX = Syntax_checker(LEXER)
                error_in='Parser'
                PARSER = Parser(LEXER)
                error_in=None
            except Exception as ee:
                line_no = '1'
                line_text = command
                tb_type, tb_text, _ = errors(ee, lineno=str(line_no),get_value_back=1)

                print(f'{Red("error catched [")} {error_in}/[stdin]/{tb_type} {Red("]")}')
                print(' ' + (' '* len(str(line_no))) + ' |')
                print(f' {Cyan(line_no)} | {Cyan(line_text)}')
                print(' ' + (' '* len(str(line_no))) + ' |')
                if '\n' in tb_text:
                    print(f'{Red("error")}:', tb_text.split('\n')[0]) # error
                    print(f'{Yellow("details")}:', '\n'.join(tb_text.split('\n')[1:])) # details
                else:
                    print(f'{Red("error")}:', tb_text) # error


            if error_in==None:
                if first_time_run_command:
                    memory=Compiler(PARSER,'[stdin]')
                    first_time_run_command = False
                else:
                    memory=Compiler(PARSER,'[stdin]',memory=memory)
        except KeyboardInterrupt:
            sys.exit(1)
        except EOFError:
            sys.exit(1)
# Compile a Kagsa Library
# kagsa -l libFile.kg -o newFile.kgl
elif (args['do'] == 'compile_lib'):
    if args['state'] != 'ok':
        sys.exit(args['message'])
    
    kg_file = args['input'][0]
    output  = args['input'][1]
    
    ## Check main file
    try:
        open(kg_file,'r')
    except:
        sys.exit(f'error : compile_lib : "{kg_file}" is not defined')

    error_in = None
    try:
        error_in='Lexer'
        LEXER = Lexer(  open(kg_file,'r').read()  )
        error_in='SyntaxChecker'
        SYNTAX = Syntax_checker(LEXER)
        error_in='Parser'
        PARSER = Parser(LEXER)
        error_in=None
    except Exception as ee:
        print(f'Error Catched in "{kg_file}", Out From {error_in} :')
        if ', line' in str(ee):
            line_no = int(re.findall(r', line (\d+)',str(ee))[0])
            line_text = open(kg_file,'r').read().split('\n')[line_no-1].strip()
        else:
            line_no = '?'
            line_text = '?'
        tb_type, tb_text, _ = errors(ee, lineno=str(line_no),get_value_back=1)

        
        print(f'{Red("error catched [")} {error_in}/{kg_file}/{tb_type} {Red("]")}')
        print(' ' + (' '* len(str(line_no))) + ' |')
        print(f' {Cyan(line_no)} | {Cyan(line_text)}')
        print(' ' + (' '* len(str(line_no))) + ' |')
        if '\n' in tb_text:
            print(f'{Red("error")}:', tb_text.split('\n')[0]) # error
            print(f'{Yellow("details")}:', '\n'.join(tb_text.split('\n')[1:])) # details
        else:
            print(f'{Red("error")}:', tb_text) # error
    
    if error_in==None:
        Compiler(PARSER,kg_file,  lib=True,    lib_name=output)
# Setup a KGL File
# kagsa --setup file.kgl
elif (args['do'] == 'setup_kgl'):
    shutil.copy(args['input'][0],Paths.__path__() + os.sep + 'libs')
    print(f'COPIED : {args["input"][0]} -> {Paths.__path__() + os.sep + "libs"}')
# UnSetup a KGL File
# kagsa --unsetup file.kgl
elif (args['do'] == 'unsetup_kgl'):
    try:
        os.remove(Paths.__path__() + os.sep + 'libs' + os.sep + args['input'][0])
        print('DELETED : ' + Paths.__path__() + os.sep + 'libs' + os.sep + args['input'][0])
    except:
        sys.exit(f'error : unsetup_kgl : "{args["input"][0]}" is not defined')
# Clear KAGSA Temps
# kagsa --cleartmp
elif (args['do'] == 'cleartmp'):
    for i in os.listdir(Paths.__path__() + os.sep + 'temp'):
        try:
            os.remove(Paths.__path__() + os.sep + 'temp' + os.sep + i)
            print(F'DELETED : {Paths.__path__() + os.sep + "temp" + os.sep + i}')
        except:None
# Check Kagsa Version
# kagsa -v
elif (args['do'] == 'check_version'):
    print(f'KAGSA version : {version}')
# Check for New Updates
# kagsa -u
# kagsa --updates
elif (args['do'] == 'check_updates'):
    try:
        version_req = requests.get('https://github.com/kagsa/kagsa/releases/latest').text
        version_req = re.findall(r'<title>(.*?)</title>',version_req)[0]
        version_req = re.findall(r'\d+\.\d+\.\d+',version_req)[0]
        newVersion=version_req.replace('.','').replace(' ','')
        oldVersion=version.replace('.','').replace(' ','')
        if int(newVersion) > int(oldVersion):
            print(f'new version {version_req}\ndownload it from Github : https://github.com/kagsa/kagsa')
        else:
            print('there is no new updates')
    except:
        print('error : check_updates : failed to check updates')
# help message
# kagsa -h
# kagsa --help
elif (args['do'] == 'help'):
    print(f'''
KAGSA PROGRAMMING LANGUAGE

kagsa [run:<filename>] [library:-l <filename> -o <output>] [version:-v, --version] [updates:-u, --updates] [help:-h, --help]
options:
    -l <filename> -o <output>  : build kgl library
    -v, --version              : check version
    -u, --updates              : check for new updates
    -h, --help                 : view this message

Read more at https://www.kagsa.org/docs/
''')
else:
    print('error : unknown command, use "-h"')


#

class أ_K_A_G_S_A:
    f_i_l_e = '[KAGSA-FILE]'
    if f_i_l_e!='[stdin]':
        x = open(f_i_l_e,'r')
        c_o_d_e = x.read()
        x.close()
    else: c_o_d_e = ''
    v_e_r_s_i_o_n = '1.1.1'
    def l_a_t_e_s_t () :
        try:
            req = requests.get('https://github.com/kagsa/kagsa/releases/latest').text
            req = re.findall(r'<title>(.*?)</title>',req)[0]
            return re.findall(r'\d+\.\d+\.\d+',req)[0]
        except:
            return ''

    

class g_e_t_E_r_r_o_r ():
    def __init__ (self) :
        self.t_e_x_t     = ''
        self.l_i_n_e_n_o = ''
        self.l_i_n_e     = ''
        self.t_y_p_e     = ''
        self.f_i_l_e     = '[KAGSA-FILE]'
        try:    errGet = globals()['ERROR']
        except: errGet = locals()['ERROR']

        if self.f_i_l_e == '[stdin]':
            self.l_i_n_e_n_o = 1
            E = errors(errGet, get_value_back=True)
            self.t_e_x_t = E[1]
            self.t_y_p_e = E[0]
            self.l_i_n_e = ''
            return
        the_tb = traceback.extract_tb(sys.exc_info()[2])
        tb_filename, tb_lineno, tb_func, tb_line = the_tb[-1]
        tb_filetype = 'py'
        # parse the error filename
        # error came from kagsa code
        if tb_filename == '<string>':
            self.f_i_l_e = KAGSA_FILE
            tb_filetype = 'kg'
        # form a library
        elif 'kgtmp' in tb_filename:
            self.f_i_l_e = tb_filename.split(os.sep)[-1].replace('_main.py','')
            self.f_i_l_e = self.f_i_l_e.replace('__','ة').replace('_','').replace('ة','_')
            self.f_i_l_e+= '.kgl'
        # something else
        else:
            data_founded = False
            for ttb in the_tb:
                tb_fn, tb_lno, tb_f, _ = ttb
                if (tb_fn == '<string>'):
                    tb_filename, tb_lineno, tb_func, _ = ttb
                    data_founded =True
                    break
            if not(data_founded):
                tb_lineno = int(re.findall(r', line (\d+)',str(errGet))[0])
            self.f_i_l_e = KAGSA_FILE
            tb_filetype = 'kg'
        
        try:
            if tb_filetype == 'kg':
                tb_lineno = KAGSA_CODES.split('\n')[tb_lineno-1]
                tb_lineno = int(re.findall(r'    # line (\d+)',tb_lineno)[-1])
                x = open(KAGSA_FILE,'r')
                file_lines = x.read().split('\n')
                x.close()
                file_lines_no = len( file_lines )
            else:
                x = open(tb_filename, 'r')
                file_lines = x.read().split('\n')
                x.close()
                file_lines_no = len( file_lines )
            #
            #
            #
            if (tb_lineno == file_lines_no) or (tb_lineno < file_lines_no):
                self.l_i_n_e = file_lines[tb_lineno-1].strip()
                self.l_i_n_e_n_o   = tb_lineno
            else:
                self.l_i_n_e       = '?'
                self.l_i_n_e_n_o   ='?'
        except:
            self.l_i_n_e       = '?'
            self.l_i_n_e_n_o   = '?'
        

        E = errors(errGet, get_value_back=True)
        self.t_e_x_t = E[1]
        self.t_y_p_e = E[0]






class IncludeError (Exception):
    pass

def INCLUDE (lib):
    try:
        # check if input is string
        if not(lib.__class__.__name__ == 'str') :
            raise IncludeError('library must be string')
        # check if input is .kgl file
        if not(lib.endswith('.kgl')):
            raise IncludeError('libarary must be ".kgl" file')
        if len(re.findall(r'[a-zA-Z_][a-zA-Z0-9_]*\.kgl|[a-zA-Z_]\.kgl', lib)) < 1 :
            raise IncludeError('filename must be writted as this syntax "[a-zA-Z_][a-zA-Z0-9_]*\.kgl|[a-zA-Z_]\.kgl"')
        if not(re.findall(r'[a-zA-Z_][a-zA-Z0-9_]*\.kgl|[a-zA-Z_]\.kgl', lib)[0] == lib):
            raise IncludeError('filename must be writted as this syntax "[a-zA-Z_][a-zA-Z0-9_]*\.kgl|[a-zA-Z_]\.kgl"')
        # try open file
        try:
            open(lib)
        except:
            Paths.getFile('libs' + os.sep + lib,'r')
        if os.sep in lib:
            lib_name = lib.split(os.sep)[-1]
        lib_name = lib.replace('.kgl','')
        # parse lib_name
        if (len(lib_name)>2) or (len(lib_name)==2):
            lib_name='_'.join(list(lib_name))
        if lib_name[0] in '1234567890' :
            lib_name='_'+lib_name
        # end parse
        # read it :

        # get "main.py"
        try:
            archive = zipfile.ZipFile(lib,'r')
        except:
            archive = zipfile.ZipFile(Paths.__path__() + os.sep + 'libs' + os.sep + lib,'r')
        try:
            KAGSA_TEMP = Paths.__path__() + os.sep + 'temp' + os.sep
            # write python library
            py_file = open(f'{KAGSA_TEMP}{lib_name}_main.py','wb')
            py_file.write(archive.read('main.py'))
            py_file.close()
            # write the kg file
            kg_file_name = [ff for ff in archive.namelist() if ff.endswith('.kg')]
            kg_file = open(f'{KAGSA_TEMP}{kg_file_name[0]}','wb')
            kg_file.write(archive.read(kg_file_name[0]))
            kg_file.close()
        except:
            KAGSA_TEMP = Paths.__path__() + os.sep
            # write python library
            py_file = open(f'{KAGSA_TEMP}{lib_name}_main.py','wb')
            py_file.write(archive.read('main.py'))
            py_file.close()
            # write the kg file
            kg_file_name = [ff for ff in archive.namelist() if ff.endswith('.kg')]
            kg_file = open(f'{KAGSA_TEMP}{kg_file_name[0]}','wb')
            kg_file.write(archive.read(kg_file_name[0]))
            kg_file.close()
        # import it
        global exec_scope
        exec_scope = {}
        #sys.path.insert(1, KAGSA_TEMP[0:-1])
        try:
            exec(f'import {lib_name}_main',exec_scope)
            exec(f'def send_to_globals () :\n\tglobal {lib_name}\n\t{lib_name} = exec_scope["{lib_name}_main"]',globals())
            send_to_globals()
        except Exception as e:
            raise IncludeError(str(e))
    except FileNotFoundError:
        raise IncludeError(f'"{lib}" is not defined')

def to_list (data):
    d = []
    for i in data:
        d.append(i)
    return d

class JumpingError (Exception):
    pass

def JUMP (lineno):
    lines = KAGSA_CODES
    lines_dict = {}
    data_founded = 0
    data = ''
    
    # Create a dict with all lines -> {'2','print("hi")    # line 2'}
    last=''
    for j in lines.split('\n'):
        no = re.findall(r'    # line (\d+)',j)[0]
        if no == last.replace('_',''):
            lines_dict[last+'_'] = j
            last = last+'_'
        else:
            lines_dict[no] = j
            last = no
        #print(last)

    if str(lineno) in lines_dict.keys():
        k,v = to_list(lines_dict.keys()), to_list(lines_dict.values())
        data_founded=1
        data = v[ k.index(str(lineno)) : ]
    else:
        up_ = int(lineno)
        down_ = int(lineno)
        while True:
            up_+=1
            down_-=1
            if str(up_) in lines_dict:
                k,v = to_list(lines_dict.keys()), to_list(lines_dict.values())
                data_founded=1
                data = v[ k.index(str(up_)) : ]
                break
            if down_ > 0:
                if str(down_) in lines_dict:
                    k,v = to_list(lines_dict.keys()), to_list(lines_dict.values())
                    data_founded=1
                    data = v[ k.index(str(down_))+1 : ]
                    break


    if data[0].startswith('\t'):
        tabs = re.findall(r'(\t+).*?',data[0])[0].count('\t')
        x=0
        for j in range(1, tabs+1):
            data.insert(x , ("\t" * x) + 'for _jump_ in [1]:')
            x+=1

    #print('\n'.join(data))
    #input()
    exec('\n'.join(data),globals())
    sys.exit(0)




class ParseError (Exception):pass

def __init__ (value,parseMemory):

    # if any keyword (except 'input') did not writted in line start
    if (value!='input') and (parseMemory[5]!=''):
        print(parseMemory[5])
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nKeywords must be in the first of line')
    if value == 'jump':
        parseMemory[5]='JUMP(|DATA-P|)'
        parseMemory[7]='jump'
    if value == 'include':
        parseMemory[5]='INCLUDE(|DATA-P|)'
        parseMemory[7]='include'
    # Add The Keyword To The Line
    if value == 'global':
        parseMemory[5]='global |DATA-P|'
        parseMemory[7]='global'
    if value == 'delvar':
        parseMemory[5]='del |DATA-P|'
        parseMemory[7]='delvar'
    if value == 'write':
        parseMemory[5]='print(|DATA|,end="")'
        parseMemory[7]='write'
    if value == 'input':
        # if it writted in var : add it and complete after it (|DATA1|)
        if ('|DATA1|' in parseMemory[5]):
            parseMemory[5]=parseMemory[5].replace('|DATA1|','input(|DATA-P|)|DATA1|')
        else:
            # if the line is empty add |DATA| to start write codes
            if parseMemory[5]=='':
                parseMemory[5]+='|DATA|'
            # if it didn't writted in var : add it and complete after it (|DATA|)
            if '|DATA|' in parseMemory[5]:
                parseMemory[5]=parseMemory[5].replace('|DATA|','input(|DATA-P|)|DATA|')
            else:
                raise ParseError(f'can\'t parsing (<file>, line {parseMemory[4]})')
    if value == 'var':
        # |DATA0|  :  Variable Name
        # |DATA1|  :  Variable items
        parseMemory[5]='|DATA0| = |DATA1|'
        parseMemory[7]='var'
    if value == 'string':
        parseMemory[5]='|DATA0| = str(|DATA1|)'
        parseMemory[7]='string'
    if value == 'int':
        parseMemory[5]='|DATA0| = int(|DATA1|)'
        parseMemory[7]='int'
    if value == 'float':
        parseMemory[5]='|DATA0| = float(|DATA1|)'
        parseMemory[7]='float'
    if value == 'dict':
        parseMemory[5]='|DATA0| = json.loads(|DATA1|)'
        parseMemory[7]='dict'
    if value == 'list':
        parseMemory[5]='|DATA0| = list(|DATA1|)'
        parseMemory[7]='list'
    
    if value == 'try':
        parseMemory[5]='try :'
        parseMemory[7]='try'
    if value == 'catch':
        # Get Error Out as 'ERROR'
        parseMemory[5]='except Exception as ERROR :'
        parseMemory[7]='catch'
    if value == 'for':
        parseMemory[5]='for |DATA| :'
        parseMemory[7]='for'
    if value == 'if':
        parseMemory[5]='if |DATA| :'
        parseMemory[7]='if'
    if value == 'elseif':
        parseMemory[5]='elif |DATA| :'
        parseMemory[7]='elseif'
    if value == 'while':
        parseMemory[5]='while |DATA| :'
        parseMemory[7]='while'
    if value == 'func':
        # |DATA_N|  :  Function Name
        parseMemory[5]='def |DATA_N| |DATA| :'
        parseMemory[7]='func'
    if value == 'class':
        parseMemory[5]='class |DATA-P| :'
        parseMemory[7]='class'
    if value == 'else':
        parseMemory[5]='else :'
        parseMemory[7]='else'
    if value =='break':
        parseMemory[5]='break'
        parseMemory[7]='break'
    if value =='continue':
        parseMemory[5]='continue'
        parseMemory[7]='continue'
    if value =='return':
        parseMemory[5]='return |DATA|'
        parseMemory[7]='return'
    
    return parseMemory


import threading,forbiddenfruit


def SetMethod (datatypes, name, func):
    for datatype in datatypes:
        forbiddenfruit.curse(datatype,name,func)

def t_o_I_n_t (value):
    try:    return int(value)
    except: raise ValueError(f'can\'t change ({value}) to int')
def t_o_S_t_r (value):
    try :  return str(value)
    except: raise ValueError(f'can\'t change ({value}) to str')
def t_o_F_l_o_a_t (value):
    try:  return float(value)
    except: raise ValueError(f'can\'t change ({value}) to float')
def i_s_I_n_t (value):
    if value.__class__.__name__ == 'int':
        return 1
    else:
        return 0
def i_s_S_t_r (value):
    if value.__class__.__name__ == 'str':
        return 1
    else:
        return 0
def i_s_F_l_o_a_t (value):
    if value.__class__.__name__ == 'float':
        return 1
    else:
        return 0

def i_s_L_i_s_t (value):
    if value.__class__.__name__ == 'list':
        return 1
    else:
        return 0

def i_s_D_i_c_t (value):
    if value.__class__.__name__ == 'dict':
        return 1
    else:
        return 0

def n_l_i_s_t (num, z_e_r_o=True):
    if z_e_r_o:
        return range(0,num+1)
    else:
        return range(1,num+1)

def d_i_r (data):
    lst = []
    dd = ' '.join(dir(data))
    #print(dir(data))
    for word in [word for word in dd.split(' ') if not(word[0].isdigit() or word[0] == '_' and not word[1].isdigit() or any([len(char) > 1 for char in word.split('_')]))]:
        lst.append( ''.join(word.replace('__','ظ').split('_')).replace('ظ','_') )
    return lst


n_l = '\n'
n_o_n_e = None
n_o_t = lambda co:not(co)

# Methods Starts from Here...
def r_e_p_l_a_c_e (DATA,v1,v2):
    if 'str' == DATA.__class__.__name__:
        return DATA.replace(v1,v2)
    else:
        raise ValueError('"replace()" function take only (str) value')
def s_p_l_i_t (DATA,value):
    if ('str' == DATA.__class__.__name__):
        return DATA.split(value)
    else:
        raise ValueError('"split()" function take only (str) value')
def e_n_d (DATA,value):
    if ('str' == DATA.__class__.__name__):
        return DATA.endswith(value)
    else:
        raise ValueError('"end()" function take only (str) value')
def s_t_a_r_t (DATA,value):
    if ('str' == DATA.__class__.__name__):
        return DATA.startswith(value)
    else:
        raise ValueError('"start()" function take only (str) value')
def s_e_a_r_c_h (DATA,value):
    if ('str' == DATA.__class__.__name__):
        return DATA.find(value)
    else:
        raise ValueError('"search()" function take only (str) value')
def u_p_c_a_s_e (DATA):
    if ('str' == DATA.__class__.__name__):
        return DATA.upper()
    else:
        raise ValueError('"upcase()" function take only (str) value')
def d_o_w_n_c_a_s_e (DATA):
    if ('str' == DATA.__class__.__name__):
        return DATA.lower()
    else:
        raise ValueError('"downcase()" function take only (str) value')
def s_t_r_i_p (DATA):
    if ('str' in DATA.__class__.__name__):
        return DATA.strip()
    else:
        raise ValueError('"delspace()" function take only (str) value')
def r_e_v_e_r_s_e (DATA):
    if ('str' == DATA.__class__.__name__) or ('list' == DATA.__class__.__name__): 
        return DATA[::-1]
    else:
        ValueError('"reverse()" function take only (str,list) value')


# List Function
def l_i_s_t (*arg):
    d = []
    for i in arg: d.append(i)
    return d
# LIST, DICT, STR
def l_e_n_g_t_h (DATA):
    if ('dict' == DATA.__class__.__name__) or ('list' == DATA.__class__.__name__) or ('str' == DATA.__class__.__name__):
        return len(DATA)
    else:
        raise ValueError('"length(DATA)" function take only (dict, list, str) value')
# LIST, DICT, STR
def g_e_t (DATA,*value):
    if (('list' == DATA.__class__.__name__)) or (('dict' == DATA.__class__.__name__)) or ('str' == DATA.__class__.__name__) :
        if len(value)>1:
            if 'dict' == DATA.__class__.__name__: raise ValueError('"get(VAR , value)" function take only (list, str) value')
            return DATA[value[0]:value[1]]
        else:
            return DATA[value[0]]
    else:
        raise ValueError('"get(VAR , *value)" function take only (dict, list, str) value')
# LIST, DICT
def a_p_p_e_n_d (DATA,*arguments):
    if 'list' == DATA.__class__.__name__:
        for i in arguments :
            DATA.append(i)
    elif 'dict' == DATA.__class__.__name__:
        DATA[arguments[0]]=arguments[1]
    else:
        raise ValueError('"append(VAR , value)" function take only (list, dict) value')
# LIST, DICT
def c_l_e_a_r (DATA):
    if (('list' == DATA.__class__.__name__)) or (('dict' in DATA.__class__.__name__)):
        DATA.clear()
        return 1
    else:
        raise ValueError('"clear(VAR)" function take only (list, dict) value')
# LIST, DICT
def d_e_l_e_t_e (DATA,value,i_d_x=False):
    if ('list' == DATA.__class__.__name__):
        if i_d_x and (i_s_I_n_t(value)):
            DATA.pop(value)
        elif i_d_x==False:
            DATA.remove(value)
        else:
            raise ValueError('"dalete(VAR, value, idx=BOOL)" if "idx" = true -> value must be int')
    elif (('dict' == DATA.__class__.__name__)):
        DATA.pop(value)
    else:
        raise ValueError('"delete(VAR , value, idx=BOOL)" function take only (list, dict) value')
    return 1
# LIST, DICT
def a_d_d (DATA,pos,value):
    if (('list' == DATA.__class__.__name__)):
        if not(i_s_I_n_t(pos)):
            raise ValueError('"add(VAR, pos, val)" pos must be int')
        DATA.insert(pos,value)
    elif (('dict' == DATA.__class__.__name__)):
        DATA[pos]=value
    else:
        raise ValueError('"add()" function take only (list, dict) value')
    return 1
# LIST
def i_n_d_e_x (DATA,value):
    if (('list' == DATA.__class__.__name__)):
        return DATA.index(value)
    else:
        raise ValueError('"index(VAR , value)" function take only (list) value')
# LIST
def a_p_p_l_i_s_t (DATA,value):
    if ('list' == DATA.__class__.__name__) and ('list' == value.__class__.__name__):
        DATA.extend(value)
    else:
        raise ValueError('"applist(VAR , value)" function take only (list) values')
    return 1
# LIST, STR
def c_o_u_n_t (DATA,value):
    if (('list' == DATA.__class__.__name__)) or (('str' == DATA.__class__.__name__)):
        return DATA.count(value)
    else:
        raise ValueError('"count(VAR , value)" function take only (list) value')
# LIST
def j_o_i_n (DATA,value):
    if (('list' == DATA.__class__.__name__)):
        return value.join(DATA)
    else:
        raise ValueError('"join(VAR , value)" function take only (list) value')

def s_o_r_t_e_d (DATA):
    if ('list' == DATA.__class__.__name__):
        return sorted(DATA)
    else:
        raise ValueError('"join(VAR , value)" function take only (list) value')

# Dict Function
def d_i_c_t (**arg):
    dct={}
    for i in arg:
        text=i.replace('__','$').replace('_','').replace('$','_')
        dct[text]=arg[i]
    return dct
# DICT
def k_e_y_s (DATA):
    if (('dict' in DATA.__class__.__name__)):
        return DATA.keys()
    else:
       raise ValueError('"keys(VAR)" function take only (dict) values')
# DICT
def v_a_l_u_e_s (DATA):
    if (('dict' in DATA.__class__.__name__)):
        return DATA.values()
    else:
        raise ValueError('"values(VAR)" function take only (dict) values')

def buildMethods ():
    SetMethod([str],'r_e_p_l_a_c_e',r_e_p_l_a_c_e)
    SetMethod([str],'s_p_l_i_t',s_p_l_i_t)
    SetMethod([str],'e_n_d',e_n_d)
    SetMethod([str],'s_t_a_r_t',s_t_a_r_t)
    SetMethod([str],'s_e_a_r_c_h',s_e_a_r_c_h)
    SetMethod([str],'u_p_c_a_s_e',u_p_c_a_s_e)
    SetMethod([str],'d_o_w_n_c_a_s_e',d_o_w_n_c_a_s_e)
    SetMethod([str],'s_t_r_i_p',s_t_r_i_p)
    SetMethod([str,list,dict],'l_e_n_g_t_h',l_e_n_g_t_h)
    SetMethod([str,list],'r_e_v_e_r_s_e',r_e_v_e_r_s_e)
    SetMethod([str,list,dict],'g_e_t',g_e_t)
    SetMethod([list,dict],'a_p_p_e_n_d',a_p_p_e_n_d)
    SetMethod([list,dict],'c_l_e_a_r',c_l_e_a_r)
    SetMethod([list,dict],'d_e_l_e_t_e',d_e_l_e_t_e)
    SetMethod([list,dict],'a_d_d',a_d_d)
    SetMethod([list],'i_n_d_e_x',i_n_d_e_x)
    SetMethod([list],'a_p_p_l_i_s_t',a_p_p_l_i_s_t)
    SetMethod([list],'j_o_i_n',j_o_i_n)
    SetMethod([list],'s_o_r_t_e_d',s_o_r_t_e_d)
    SetMethod([str,list],'c_o_u_n_t',c_o_u_n_t)
    SetMethod([dict],'k_e_y_s',k_e_y_s)
    SetMethod([dict],'v_a_l_u_e_s',v_a_l_u_e_s)
    #SetMethod([str],'',)
threading.Thread(target=buildMethods).start()


def A_s_s_e_r_t_i_o_n_E_R_R (text):
    raise AssertionError(text)
def A_t_t_r_i_b_u_t_e_E_R_R (text):
    raise AttributeError(text)
def E_O_F_E_R_R (text):
    raise EOFError(text)
def F_l_o_a_t_i_n_g_P_o_i_n_t_E_R_R (text):
    raise FloatingPointError(text)
def G_e_n_e_r_a_t_o_r_E_x_i_t (text):
    raise GeneratorExit(text)
def I_m_p_o_r_t_E_R_R (text):
    raise ImportError(text)
def I_n_d_e_x_E_R_R (text):
    raise IndexError(text)
def K_e_y_E_R_R (text):
    raise KeyError(text)
def K_e_y_b_o_a_r_d_I_n_t_e_r_r_u_p_t (text):
    raise KeyboardInterrupt(text)
def M_e_m_o_r_y_E_R_R (text):
    raise MemoryError(text)
def N_a_m_e_E_R_R (text):
    raise NameError(text)
def N_o_t_I_m_p_l_e_m_e_n_t_e_d_E_R_R (text):
    raise NotImplementedError(text)
def O_S_E_R_R (text):
    raise OSError(text)
def O_v_e_r_f_l_o_w_E_R_R (text):
    raise OverflowError(text)
def R_e_f_e_r_e_n_c_e_E_R_R (text):
    raise ReferenceError(text)
def R_u_n_t_i_m_e_E_R_R (text):
    raise RuntimeError(text)
def S_t_o_p_I_t_e_r_a_t_i_o_n (text):
    raise StopIteration(text)
def S_y_n_t_a_x_E_R_R (text):
    raise SyntaxError(text)
def I_n_d_e_n_t_a_t_i_o_n_E_R_R (text):
    raise IndentationError(text)
def T_a_b_E_R_R (text):
    raise TabError(text)
def S_y_s_t_e_m_E_R_R (text):
    raise SystemError(text)
def S_y_s_t_e_m_E_x_i_t (text):
    raise SystemExit(text)
def T_y_p_e_E_R_R (text):
    raise TypeError(text)
def U_n_b_o_u_n_d_L_o_c_a_l_E_R_R (text):
    raise UnboundLocalError(text)
def U_n_i_c_o_d_e_E_R_R (text):
    raise UnicodeError(text)
def U_n_i_c_o_d_e_E_n_c_o_d_e_E_R_R (text):
    raise UnicodeEncodeError(text)
def U_n_i_c_o_d_e_D_e_c_o_d_e_E_R_R (text):
    raise UnicodeDecodeError(text)
def U_n_i_c_o_d_e_T_r_a_n_s_l_a_t_e_E_R_R (text):
    raise UnicodeTranslateError(text)
def V_a_l_u_e_E_R_R (text):
    raise ValueError(text)
def Z_e_r_o_D_i_v_i_s_i_o_n_E_R_R (text):
    raise ZeroDivisionError(text)
def N_e_w_E_R_R (name):
    ss={}
    exec(f'class {name} (Exception):\n\tpass\ndef rrr (text):\n\traise {name}(text)',ss)
    return ss['rrr']


import sys,platform,base64,os,requests,re,json,random,time,datetime,keyboard,math
try:
    from .methods import *
except:
    from methods import *

#

P_A_T_H = sys.path


class W_e_b:
    class HTMLError (Exception):pass
    def s_c_r_i_p_t (data):
        return f'<script>{data}</script>'
    def b_o_x (data):
        data = data.replace('"','\\"')
        print(f'<script>alert("{data}")</script>')
    class E_l_e_m_e_n_t:
        def __init__ (self,tagName,n,innerHTML,**attrs):
            self.t_a_g_N_a_m_e = tagName
            if not(n in [1,2,'1','2']):
                raise W_e_b.HTMLError('Element(tagName, n, innerHTML, **attrs) n must be 1 or 2')
            self.n = n
            self.a_t_t_r_s = attrs
            self.i_n_n_e_r_H_T_M_L = innerHTML
        def __str__ (self):
            d = ''
            if self.n == 1:
                d = f'<{self.t_a_g_N_a_m_e} _DATA_>'
            else:
                d = f'<{self.t_a_g_N_a_m_e} _DATA_>{self.i_n_n_e_r_H_T_M_L}</{self.t_a_g_N_a_m_e}>'
            for i,j in self.a_t_t_r_s.items():
                j =j.replace('"','\\"')
                i=i.replace('__','ة').replace('_','').replace('ة','_')
                d=d.replace('_DATA_',f'{i}="{j}" _DATA_')
            d=d.replace(' _DATA_','')
            return d
    class i_n_p_u_t_s:
        inp = {}
        def s_e_t (selector,htmlclass):
            W_e_b.i_n_p_u_t_s.inp[selector] = htmlclass
        def g_e_t (selector):
            if selector in W_e_b.i_n_p_u_t_s.inp.keys():
                return W_e_b.i_n_p_u_t_s.inp[selector]
            else:
                raise W_e_b.HTMLError(f"{selector} is not denfined")
        def e_d_i_t (selector,**data):
            o = "<script>_DATA_</script>"
            for i,j in data.items():
                i=i.replace('__','ة').replace('_','').replace('ة','_')
                j = j.replace('"','\\"')
                o = o.replace('_DATA_', f'document.querySelector("{selector}").{i} = "{j}";_DATA_')
            o = o.replace('_DATA_','')
            print(o)
        


class K_e_y_b_o_a_r_d:
    class KeyboardError (Exception) :pass
    def w_a_i_t (text):
        try:
            keyboard.wait(text)
        except Exception as ee : raise K_e_y_b_o_a_r_d.KeyboardError(str(ee))
    def w_r_i_t_e (text):
        try:
            keyboard.write(text)
        except Exception as ee : raise K_e_y_b_o_a_r_d.KeyboardError(str(ee))
    def n_e_w_H_o_t_k_e_y (text, func):
        try:
            keyboard.add_hotkey(text, func)
        except Exception as ee : raise K_e_y_b_o_a_r_d.KeyboardError(str(ee))
    class r_e_c_o_r_d:
        def __init__ (self,u_n_t_i_l=''):
            try:
                self.g_e_t = []
                self.recorded = []
                keyboard.hook(self.recorded.append)
                keyboard.wait(u_n_t_i_l)
                keyboard.unhook(self.recorded.append)
                for i in self.recorded:
                    self.g_e_t.append(str(i).replace('KeyboardEvent(','')[0:-1])
            except Exception as ee : raise K_e_y_b_o_a_r_d.KeyboardError(str(ee))
        def p_l_a_y_A_l_l (self,s_p_e_e_d=0.5):
            keyboard.play(self.recorded, speed_factor = s_p_e_e_d)
    def p_r_e_s_s (text):
        try:
            keyboard.press_and_release(text)
        except Exception as ee : raise K_e_y_b_o_a_r_d.KeyboardError(str(ee))


class M_a_t_h:
    def a_c_o_s (*arg) :
        return math.acos(*arg)
    def a_c_o_s_h (*arg) :
        return math.acosh(*arg)
    def a_s_i_n (*arg) :
        return math.asin(*arg)
    def a_s_i_n_h (*arg) :
        return math.asinh(*arg)
    def a_t_a_n (*arg) :
        return math.atan(*arg)
    def a_t_a_n_2 (*arg) :
        return math.atan2(*arg)
    def a_t_a_n_h (*arg) :
        return math.atanh(*arg)
    def c_e_i_l (*arg) :
        return math.ceil(*arg)
    def c_o_m_b (*arg) :
        return math.comb(*arg)
    def c_o_p_y_s_i_g_n (*arg) :
        return math.copysign(*arg)
    def c_o_s (*arg) :
        return math.cos(*arg)
    def c_o_s_h (*arg) :
        return math.cosh(*arg)
    def d_e_g_r_e_e_s (*arg) :
        return math.degrees(*arg)
    def d_i_s_t (*arg) :
        return math.dist(*arg)
    def e (*arg) :
        return math.e(*arg)
    def e_r_f (*arg) :
        return math.erf(*arg)
    def e_r_f_c (*arg) :
        return math.erfc(*arg)
    def e_x_p (*arg) :
        return math.exp(*arg)
    def e_x_p_m_1 (*arg) :
        return math.expm1(*arg)
    def f_a_b_s (*arg) :
        return math.fabs(*arg)
    def f_a_c_t_o_r_i_a_l (*arg) :
        return math.factorial(*arg)
    def f_l_o_o_r (*arg) :
        return math.floor(*arg)
    def f_m_o_d (*arg) :
        return math.fmod(*arg)
    def f_r_e_x_p (*arg) :
        return math.frexp(*arg)
    def f_s_u_m (*arg) :
        return math.fsum(*arg)
    def g_a_m_m_a (*arg) :
        return math.gamma(*arg)
    def g_c_d (*arg) :
        return math.gcd(*arg)
    def h_y_p_o_t (*arg) :
        return math.hypot(*arg)
    def i_n_f (*arg) :
        return math.inf(*arg)
    def i_s_c_l_o_s_e (*arg) :
        return math.isclose(*arg)
    def i_s_f_i_n_i_t_e (*arg) :
        return math.isfinite(*arg)
    def i_s_i_n_f (*arg) :
        return math.isinf(*arg)
    def i_s_n_a_n (*arg) :
        return math.isnan(*arg)
    def i_s_q_r_t (*arg) :
        return math.isqrt(*arg)
    def l_c_m (*arg) :
        return math.lcm(*arg)
    def l_d_e_x_p (*arg) :
        return math.ldexp(*arg)
    def l_g_a_m_m_a (*arg) :
        return math.lgamma(*arg)
    def l_o_g (*arg) :
        return math.log(*arg)
    def l_o_g_1_0 (*arg) :
        return math.log10(*arg)
    def l_o_g_1_p (*arg) :
        return math.log1p(*arg)
    def l_o_g_2 (*arg) :
        return math.log2(*arg)
    def m_o_d_f (*arg) :
        return math.modf(*arg)
    def n_a_n (*arg) :
        return math.nan(*arg)
    def n_e_x_t_a_f_t_e_r (*arg) :
        return math.nextafter(*arg)
    def p_e_r_m (*arg) :
        return math.perm(*arg)
    def p_i (*arg) :
        return math.pi(*arg)
    def p_o_w (*arg) :
        return math.pow(*arg)
    def p_r_o_d (*arg) :
        return math.prod(*arg)
    def r_a_d_i_a_n_s (*arg) :
        return math.radians(*arg)
    def r_e_m_a_i_n_d_e_r (*arg) :
        return math.remainder(*arg)
    def s_i_n (*arg) :
        return math.sin(*arg)
    def s_i_n_h (*arg) :
        return math.sinh(*arg)
    def s_q_r_t (*arg) :
        return math.sqrt(*arg)
    def t_a_n (*arg) :
        return math.tan(*arg)
    def t_a_n_h (*arg) :
        return math.tanh(*arg)
    def t_a_u (*arg) :
        return math.tau(*arg)
    def t_r_u_n_c (*arg) :
        return math.trunc(*arg)
    def u_l_p (*arg) :
        return math.ulp(*arg)


class R_a_n_d_o_m:
    def s_t_r (length, a_b_c=True, c_a_p=True, n_u_m=True, a_d_d=''):
        chars = ''
        if a_b_c:
            chars+='qwertyuioplkjhgfdsazxcvbnm'
        if c_a_p:
            chars+='QWERTYUIOPLKJHGFDSAZXCVBNM'
        if n_u_m:
            chars+='0123456789'
        chars+=a_d_d
        chars = chars * 2
        return ''.join(random.sample(chars,length))
    def i_n_t (s,e):
        return random.randint(s,e)
    def o_n_e_O_f (data):
        return random.choice(data)


class C_o_d_e:
    class CodeError (Exception):
        pass
    class u_t_f___8 :
        def d_e_c_o_d_e (data):
            if data.__class__.__name__ != 'bytes':
                raise C_o_d_e.CodeError('utf-8 input data must be bytes')
            return data.decode('utf-8')
        def e_n_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('utf-8 input data must be string')
            return data.encode('utf-8')
    class b_a_s_e_6_4 :
        def d_e_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('base64 input data must be string')
            base64_byte = data.encode('ascii')
            ascii_data = base64.b64decode(base64_byte)
            return ascii_data.decode('ascii')
        def e_n_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('base64 input data must be string')
            encoded_ascii = data.encode('ascii')
            base64_byte = base64.b64encode(encoded_ascii)
            return base64_byte.decode('ascii')
    class h_e_x :
        def e_n_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('hex input data must be string')
            data=data.encode('utf-8')
            data2 = data.hex()
            return data2
        def d_e_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('hex input data must be string')
            data2 = bytes.fromhex(data)
            data2 = data2.decode('ascii')
            return data2
    class b_i_n_a_r_y :
        def e_n_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('binary input data must be string')
            return "".join(format(ord(i),'08b')for i in data)
        def d_e_c_o_d_e (data):
            if data.__class__.__name__ != 'str':
                raise C_o_d_e.CodeError('binary input data must be string')
            out=''
            d2=list(data)
            end=[]
            while len(d2)!=0:
                end.append(''.join(d2[0:8]))
                d2=d2[8:]
            for value in end:
                an_integer = int(value,2)
                ascii_char = chr(an_integer)
                out+=ascii_char
            return out


class J_S_O_N:
    class JsonError (Exception):
        pass
    class p_a_r_s_e :
        def __init__(self,data):
            if data.__class__.__name__ == 'str' :
                data = J_S_O_N.t_o_D_i_c_t( data )
            if data.__class__.__name__ == 'dict' :
                pass
            else:
                raise J_S_O_N.JsonError('JSON.pasre() take only str/dict value')

            for i in data.keys():
                if data[i].__class__.__name__ == 'dict':
                    try :     exec(f'self.{"_".join(list(i))} = J_S_O_N.p_a_r_s_e({data[i]})')
                    except:  raise J_S_O_N.JsonError(f'value error : "{i}"')
                else:
                    if data[i].__class__.__name__ == 'str':
                        data[i] = f'"{data[i]}"'
                    try :     exec(f'self.{"_".join(list(i))} = {data[i]}')
                    except:  raise J_S_O_N.JsonError(f'value error : "{i}"')
    def u_n_P_a_r_s_e (data):
        if not('p_a_r_s_e' in data.__class__.__name__):
            raise J_S_O_N.JsonError('JSON.unPasre() take only "JSON.parse()" value')
        outdict = {}
        for i in dir(data):
            if len(re.findall(r'__[a-zA-Z0-9_]*__',i)) > 0 :
                continue
            i2 = i.replace('__','ة').replace('_','').replace('ة','_')
            exec(f'''if J_S_O_N.p_a_r_s_e == data.{i}.__class__:\n\toutdict["{i2}"] = J_S_O_N.u_n_P_a_r_s_e(data.{i})\nelse:\n\texec(f'outdict["{i2}"] = data.{i}')''')
        return outdict
    def t_o_J_s_o_n (data):
        try:
            return json.dumps(data)
        except Exception as e:
            raise J_S_O_N.JsonError(str(e))
    def t_o_D_i_c_t (data):
        try:
            return json.loads(data)
        except Exception as e:
            raise J_S_O_N.JsonError(str(e))


class H_T_T_P:
    def __init__ (self):
        self.session=requests.Session()
        self.url=''
        self.method=''
    class HTTPRequestError (Exception):
        'For Any Request Error'
        pass
    class response:
        def __init__ (self,data):
            self.u_r_l            =  data.url
            self.r_e_d_i_r_e_c_t  =  data.is_redirect
            self.c_o_o_k_i_e_s    =  data.cookies.get_dict()
            self.h_e_a_d_e_r_s    =  data.headers
            self.t_e_x_t          =  data.text
            self.c_o_n_t_e_n_t    =  data.content
            self.c_o_d_e          =  data.status_code
    def h_e_a_d_e_r_s (self,data):
        self.session.headers.update(data)
    def e_n_c_o_d_i_n_g (self,data):
        self.session.encoding = data
    def a_u_t_h (self,data):
        self.session.auth=data
    def c_o_o_k_i_e_s (self,data):
        self.session.cookies.update(data)
    def U_R_L (self,*url):
        if len(url) < 1 :
            raise self.HTTPRequestError('Enter a Url in "HTTP().URL( URL_HERE )" function')
        self.url='/'.join(url)
    def m_e_t_h_o_d (self,data):
        if data.upper().strip() in ['GET','POST','PUT','DELETE','HEAD','PATCH','OPTIONS']:
            self.method=data.upper().strip()
        else:
            raise self.HTTPRequestError('Unknown Method You Can Use (GET, POST)')
    def s_e_n_d (self,*data):
        try:
            if self.method=='': raise self.HTTPRequestError('Please Set HTTP Request Method "HTTP().method( METHOD_HERE )"')
            if self.url=='':    raise self.HTTPRequestError('Please Set Request URL "HTTP().url( URL_HERE )"')
            
            if self.method == 'GET':
                if len(data) >= 1 :   req=self.session.get(self.url, data=data[0])
                else:                 req=self.session.get(self.url)
            if self.method == 'POST':
                if len(data) >= 1 :   req=self.session.post(self.url, data=data[0])
                else:                 req=self.session.post(self.url)
            if self.method == 'PUT':
                if len(data) >= 1 :   req=self.session.put(self.url, data=data[0])
                else:                 req=self.session.put(self.url)
            if self.method == 'DELETE':
                if len(data) >= 1 :   req=self.session.delete(self.url, data=data[0])
                else:                 req=self.session.delete(self.url)
            if self.method == 'HEAD':
                if len(data) >= 1 :   req=self.session.head(self.url, data=data[0])
                else:                 req=self.session.head(self.url)
            if self.method == 'PATCH':
                if len(data) >= 1 :   req=self.session.patch(self.url, data=data[0])
                else:                 req=self.session.patch(self.url)
            if self.method == 'OPTIONS':
                if len(data) >= 1 :   req=self.session.options(self.url, data=data[0])
                else:                 req=self.session.options(self.url)
            
            return self.response(req)
        except Exception as e:
            raise self.HTTPRequestError(str(e))


class R_e_g_e_x :
    def __init__ (self,reStr):
        self.re_str=fr'{reStr}'
    def i_s_M_a_t_c_h (self,string):
        if re.search(self.re_str,string):
            return True
        else:
            return False
    def f_i_n_d_A_l_l (self,string):
        return re.findall(self.re_str,string)
    def i_n_d_e_x_S_t_a_r_t (self,string):
        try:
            return re.search(self.re_str,string).start()
        except:
            return -1
    def i_n_d_e_x_E_n_d (self,string):
        try:
            return re.search(self.re_str,string).end()
        except:
            return -1
    def s_p_l_i_t (self,string,t_i_l_l=None):
        if t_i_l_l==None:
            return re.split(self.re_str,string)
        else:
            return re.split(self.re_str,string,t_i_l_l)
    def r_e_p_l_a_c_e(self,string,join):
        return re.sub(self.re_str, join, string)
    

class S_y_s_t_e_m:
    a_r_g_v = sys.argv[1:]
    def r_e_a_d ():
        return sys.stdin.readline()
    class w_r_i_t_e :
        def o_u_t (*text):
            for i in text : sys.stdout.write(i)
            return 1
        def e_r_r (*text):
            print(*text, file = sys.stderr)
            return 1
    def e_x_i_t (text):
        sys.exit(text)
    def c_m_d (command):
        os.system(command)
        return 0
    def e_x_e_c (command):
        popen = os.popen(command)
        return popen.read()
    def p_a_t_h ():
        return os.getcwd()
    def i_n_f_o_r_m_a_t_i_o_n_s ():
        UNAME = platform.uname()
        out = {
            'system':UNAME.system,
            'node':UNAME.node,
            'release':UNAME.release,
            'version':UNAME.version,
            'machine':UNAME.machine,
            'processor':UNAME.processor,
        }
        return out


class F_i_l_e:
    class R_e_a_d:
        def __init__ (self,filename,u_t_f___8=0):
            if u_t_f___8:
                self.F = open (filename,'r',encoding='utf-8')
            else:
                self.F = open (filename,'r')
        def g_e_t_T_e_x_t (self):
            return self.F.read()
        def e_n_d (self):
            self.F.close()
    class R_e_a_d_B_y_t_e_s:
        def __init__ (self,filename):
            self.F = open (filename,'rb')
        def g_e_t (self):
            return self.F.read()
        def e_n_d (self):
            self.F.close()
    class W_r_i_t_e:
        def __init__ (self,filename,u_t_f___8=0):
            if u_t_f___8:
                self.F = open (filename,'w',encoding='utf-8')
            else:
                self.F = open (filename,'w')
        def w_r_i_t_e (self,*arguments):
            for arg in arguments:
                self.F.write(arg)
        def e_n_d (self):
            self.F.close()
    class A_d_d:
        def __init__ (self,filename,u_t_f___8=0):
            if u_t_f___8:
                self.F = open (filename,'a',encoding='utf-8')
            else:
                self.F = open (filename,'a')
        def w_r_i_t_e (self,*arguments):
            for arg in arguments:
                self.F.write(arg)
        def e_n_d (self):
            self.F.close()


class T_i_m_e:
    class get_time_data :
        def __init__ (self,obj):
            self.y_e_a_r  = obj.tm_year
            self.m_o_n  = obj.tm_mon
            self.m_d_a_y  = obj.tm_mday
            self.h_o_u_r  = obj.tm_hour
            self.m_i_n  = obj.tm_min
            self.s_e_c  = obj.tm_sec
            self.w_d_a_y  = obj.tm_wday
            self.y_d_a_y  = obj.tm_yday
            self.i_s_d_s_t  = obj.tm_isdst
    def e_p_o_c_h () :
        return time.time()
    def a_E_p_o_c_h (num, o_b_j = False) :
        if o_b_j:
            return T_i_m_e.get_time_data( time.localtime(num) )
        else:
            return time.ctime(num)
    def s_l_e_e_p (num) :
        time.sleep(num)
    def n_o_w (o_b_j = False) :
        if o_b_j:
            return T_i_m_e.get_time_data( time.localtime( time.time() ) )
        else:
            return time.ctime( time.time() )
    class c_o_u_n_t:
        def __init__ (self,o_b_j=False):
            pass
            self.obj = o_b_j
            self.time = 0
        def s_t_a_r_t (self):
            self.time =  datetime.datetime.now()
        def e_n_d (self):
            if self.obj:
                obj = datetime.datetime.now() - self.time
                self.d_a_y_s = obj.days
                self.m_i_n = obj.min
                self.s_e_c_o_n_d_s = obj.seconds
                self.m_i_c_r_o_s_e_c_o_n_d_s = obj.microseconds
                self.t_o_t_a_l___s_e_c_o_n_d_s = obj.total_seconds()
            else:
                self.g_e_t = str(datetime.datetime.now() - self.time)


import re

def __init__ (value,parseMemory):

    # catch errors
    if (parseMemory[7] == 'else'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to else command')
    if (parseMemory[7] == 'catch'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to catch command')
    if (parseMemory[7] == 'try'):
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\ncan\'t add any arguments to try command')
    
    # if |DATA-P| is empty : close it
    if '|DATA-P|' in parseMemory[5]:
        parseMemory[5]=parseMemory[5].replace('|DATA-P|','')
    # this mean the var is opened with assign
    if ('|DATA1|' in parseMemory[5]) and (parseMemory[6] == False):
        parseMemory[6] = True
        parseMemory[5] = parseMemory[5].replace('|DATA0|','')
    # if creat/call function in the line
    elif (parseMemory[5].startswith('def ') or len(re.findall(r'[a-zA-Z0-9]* *\(|[a-zA-Z0-9] *\(',parseMemory[5]))>0) and (parseMemory[5].count('(') > parseMemory[5].count(')')) :
        #print(parseMemory[5])
        if '|DATA1|' in parseMemory[5]:
            parseMemory[5]=parseMemory[5].replace('|DATA1|','=|DATA1|')
        else:
            parseMemory[5]=parseMemory[5].replace('|DATA|','=|DATA|')
    elif (parseMemory[5].count('(') == parseMemory[5].count(')')):
        parseMemory[6] = True
        parseMemory[5] = parseMemory[5].replace('|DATA|','= |DATA1|')
    else:
        raise SyntaxError(f'invalid syntax (<file>, line {parseMemory[4]})\nUnknown place of assign')

    return parseMemory


from .parse_endl import __init__ as parse_endl

def __init__ (value,parseMemory):

    parseMemory[5] = parseMemory[5].replace('|DATA-P|','')
    if value=='{':
        parseMemory[1]=True
    else:
        parseMemory[1] = False
        if parseMemory[0]==0:
            #print(parsed_input)
            raise SyntaxError('invalid syntax (<file>, line '+ str(parseMemory[4]) +')\nunknown place for }')
        parseMemory[0]-=1
    
    parseMemory = parse_endl('\n',parseMemory,addline=False)
    return parseMemory


from setuptools import setup, find_packages

__name__ = "kagsa"
__version__ = "1.2.0"
__author__ = "KAGSA PROGRAMMING LANGUAGE"
__author_email__ = "admin@kagsa.org"
__date__ = "2023-6-6"


readme = open("README.md", "r", encoding="utf-8")
long_description =  readme.read()
reqFile=open("requirements.txt", "r")
requirements = reqFile.read().split('\n')

readme.close()
reqFile.close()

setup(
    name=__name__,
    version=__version__,
    description='PROGRAMMING LANGUAGE',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://www.kagsa.org/',
    author=__author__,
    author_email=__author_email__,

    project_urls={
      'Source': 'https://github.com/kagsa/kagsa',
      'Report Bugs': 'https://github.com/kagsa/kagsa/issues',
      'Download': 'https://pypi.org/project/kagsa/#files',
      'Documentation': 'https://github.com/kagsa/kagsa/blob/master/README.md',
      'Website' : 'https://www.kagsa.org/'
    },

    license='MIT',

    keywords=[
        'programming', 'language', 'programming language',
        'kagsa', 'lang', 'kg', 'KG', 'KAGSA'
       ],

    packages=find_packages(),

    include_package_data=True,

    package_data={'' : ['built_modules.py','errors.py','methods.py','parse_id.py','parse_string.py','parser.py']},

    install_requires= requirements,

    entry_points={
        'console_scripts': [
                "kagsa=kagsasrc.__main__:main",
                "kg=kagsasrc.__main__:main"
        ]
	}
)